﻿if select(2, UnitClass('player')) ~= "SHAMAN" then
	return
end

TotemBar = CreateFrame("Frame")
TotemBar.__index = TotemBar

local LBF = LibStub("LibButtonFacade", true)
	
function TotemBar.Create()
	local bar = CreateFrame("Frame", "YataBarGrip", UIParent, "SecureFrameTemplate, SecureHandlerAttributeTemplate")
	setmetatable(bar, TotemBar)
	
	bar:New()
	
	return bar
end

function TotemBar:New()
	self:Init()

	Yata:InitTotems()
	Yata:InitTotemSets()
	
	self.Set = Yata:GetTotemSet(nil) -- Retrieve the current set
	
	-- Need to get the group order then iterate through the other items
	for k, spellgroup in ipairs(self.Set["GroupOrder"]) do
		
		-- Create the header and proxy for the group
		local header = Header:Create(spellgroup, self)
		header:Show()
		header:SetParent(self)
		self.Groups[spellgroup].Header = header
	
		local proxy = Yata.Button:CreateTotemProxy(spellgroup,self)
		proxy:SetPoint("TOPLEFT",proxy.Parent,"TOPLEFT",-50,(k-1)*36)
		proxy:Hide();
		proxy:SetParent(self)
		self.Groups[spellgroup].Proxy = proxy
		
		if LBF then
			proxy.LBFButtonData = {Button = proxy, Icon = proxy.Icon, HotKey = proxy.HotKey, Cooldown = proxy.Cooldown}
			self.LBFGroup:AddButton(proxy, proxy.LBFButtonData)				
		end		
		
		for l, totemname in ipairs(self.Set[spellgroup]) do

			-- Create the buttons for each spell
			local totem = Yata:GetTotem(totemname)	
			
			local button = Yata.Button:Create(totem.Name,self.Groups[totem.SpellGroup].Header, totem)
					
			if LBF then
				button.LBFButtonData = {Button = button, Icon = button.Icon, HotKey = button.HotKey, Cooldown = button.Cooldown}
				self.LBFGroup:AddButton(button, button.LBFButtonData)				
			end			
			
			button:SetAttribute("position", l-1)
			
			table.insert(self.Groups.All, button)
			table.insert(self.Groups[totem.SpellGroup].Buttons,button)
		end
	end
		
	self:SetProxyBinding()
	self:SkinButtons()
	self:SetButtonSwapKey()
	self:SetButtonNoSwapKey()
	self:SetAutoButtonSwap(Yata.db.profile.AutoButtonSwap)
	self:ConfigureBarDisplay()
		
	self:PositionButtons()
	
	Yata:InitTimer()

	self:EditMacro(true,nil,nil)
end

function TotemBar:ApplyTotemSet(name)

	local set = Yata:GetTotemSet(name)

	for k, spellGroup in ipairs(set.GroupOrder) do -- Loop through the spellgroups and apply position to the headers
		self.Groups[spellGroup].Header:SetAttribute("position", k-1)
	
		for j, spellName in ipairs(set[spellGroup]) do -- Loop through each spellgroup and apply position to the buttons
			for l, button in ipairs(self.Groups[spellGroup].Buttons) do
				if button.Totem.Name == spellName then
					button:SetAttribute("newposition", j-1 )
				end
			end
		end
	end
	
	if (set["HiddenSpells"]) then
		for j, spellName in ipairs(set["HiddenSpells"]) do -- Loop through each button and set to hidden if hidden
			for l, button in ipairs(self.Groups.All) do
				if button.Totem.Name == spellName then
					button:SetAttribute("hidden", true)
				end
			end
		end
	end
end

function TotemBar:Destroy()
	self:SetParent(nil)
	self.Name = nil
	self:Hide()
		
	for k,spellgroup in ipairs(SpellGroups) do
			
		if self.Groups[spellgroup].Header then		
			self.Groups[spellgroup].Header:SetParent(nil)
			self.Groups[spellgroup].Header:Hide()
		end			
	
		if(self.Groups[spellgroup].Proxy) then
			self.Groups[spellgroup].Proxy:SetParent(nil)
			self.Groups[spellgroup].Proxy:Hide()
		end
	end
	
	for m,button in ipairs(self.Groups.All) do
		if(button) then
			button:SetParent(nil)
			button:Hide()
		end
	end
end

function TotemBar:Init()
	self.Name = "YataBarGrip"
	self.Parent = UIParent
	self:SetHeight(36)
	self:SetWidth(0)
	self.Texture = self:CreateTexture()
	self.Texture:SetTexture(0, 0, 0.5, 0.5)
	self.Texture:SetAllPoints(self)	
	self:RegisterForDrag( "LeftButton" )
	self:EnableMouse(true)
	self:SetMovable(true)
	
	self:SetScript("OnDragStart", function()
		if(not Yata.db.profile.Locked) then
			this:StartMoving()
		end
	end)
	self:SetScript("OnDragStop", function()
		this:StopMovingOrSizing()
		self:SaveBarConfig()
	end)	
	
	self.Groups = {["All"] = {},
							["Earth"] = { ["Header"] = nil, ["Buttons"] = {}, ["Proxy"] = nil}, 
							["Fire"] = {["Header"] = nil, ["Buttons"] = {}, ["Proxy"] = nil }, 
							["Water"] = {["Header"] = nil, ["Buttons"] = {}, ["Proxy"] = nil }, 
							["Air"] = {["Header"] = nil, ["Buttons"] = {}, ["Proxy"] = nil }, 
							[SPELL_GROUP_CALL] = {["Header"] = nil, ["Buttons"] = {}, ["Proxy"] = nil }, 
							[SPELL_GROUP_IMBUE] = {["Header"] = nil, ["Buttons"] = {}, ["Proxy"] = nil }}
	
	if LBF then
		self.LBFGroup = LBF:Group("Yata")
		self.LBFGroup.SkinID = Yata.db.profile.ButtonSkin
		LBF:RegisterSkinCallback("Yata",self.SkinChanged,self)		
	end	
	
	self:RegisterEvent("SPELL_UPDATE_COOLDOWN", "BarUpdateHandler")
	self:RegisterEvent("UPDATE_MULTI_CAST_ACTIONBAR", "UpdateCallIndicators")
	
	self:LoadPosition()
end

function TotemBar:SkinChanged(SkinID,Gloss,Backdrop,Group,Button,Colors)
	Yata.db.profile.ButtonSkin = SkinID
	self:ConfigureBarDisplay()
end

function TotemBar:BarUpdateHandler()
	if Yata.db.profile.TimerType == 1 then
		local totem = Yata.TotemData[GetSpellInfo(8071)] -- Stoneskin Totem
		
		local start, duration, enable = GetSpellCooldown(8071)
		for k,v in ipairs(self.Groups.All) do
			CooldownFrame_SetTimer(v.Cooldown, start, duration, enable)
		end
	end
end

function TotemBar:PositionButtons()
	for j, spellgroup in ipairs(SpellGroups) do
		for k, button in ipairs(self.Groups[spellgroup].Buttons) do
			button:SetAttribute("newposition", k-1)
		end
	end
end

function TotemBar:SetPopOutScale(value)
	for i,spellgroup in ipairs(SpellGroups) do
		self.Groups[spellgroup].Header:SetAttribute("popoutscale", value)
	end
end

function TotemBar:SkinButtons(skin)
	Yata.db.profile.ButtonSkin = skin or Yata.db.profile.ButtonSkin
	if LBF and self.LBFGroup then
		self.LBFGroup:Skin(skin,0,true)
	end
end

function TotemBar:SetAutoButtonSwap(autobutton)	
	Yata.db.profile.AutoButtonSwap = autobutton
	for i,b in ipairs(self.Groups.All) do
		b:SetOnClickScript(Yata.db.profile.AutoButtonSwap)
	end
end

function TotemBar:SetButtonSetCallKey(key)	
	Yata.db.profile.ButtonSetCallKey = key
	for i,b in ipairs(self.Groups.All) do
		b:SetAttribute("setcallkey", Yata.db.profile.ButtonSetCallKey)
	end
end

function TotemBar:SetButtonSwapKey(key)
	Yata.db.profile.ButtonSwapKey = key or Yata.db.profile.ButtonSwapKey
	for i,b in ipairs(self.Groups.All) do
		b:SetAttribute("swapkey",Yata.db.profile.ButtonSwapKey)
	end
end

function TotemBar:SetButtonNoSwapKey(key)
	Yata.db.profile.ButtonNoSwapKey = key or Yata.db.profile.ButtonNoSwapKey
	for i,b in ipairs(self.Groups.All) do
		b:SetAttribute("noswapkey",Yata.db.profile.ButtonNoSwapKey)
	end
end

function TotemBar:LoadPosition()
	if Yata.db.profile.Locked then
		self.Texture:Hide()
	end	
	local x,y,scale
	if Yata.db.profile.BarPosition then
		x = Yata.db.profile.BarPosition.x
		y = Yata.db.profile.BarPosition.y
	else
		self:ClearAllPoints()
		self:SetPoint( "CENTER", WorldFrame, "CENTER", 0,0 )
		x = self:GetLeft()
		y = self:GetBottom()
	end	
	self:ClearAllPoints()
	self:SetPoint("BOTTOMLEFT", UIParent,"BOTTOMLEFT", x, y)
	self:SetScale(Yata.db.profile.BarScale)	
end

function TotemBar:SaveBarConfig()
	if not Yata.db.profile.BarPosition then 
		Yata.db.profile.BarPosition = {}
	end
	
	local x = self:GetLeft()
	local y = self:GetBottom()
	
	Yata.db.profile.BarPosition.x = x
	Yata.db.profile.BarPosition.y = y
end

function TotemBar:Unlock()
	Yata.db.profile.Locked = false
	self.Texture:Show()
	for k,v in ipairs(SpellGroups) do
		self.Groups[v].Header:Hide()
	end
	Yata:ShowTimerAnchors()
end

function TotemBar:Lock()
	Yata.db.profile.Locked = true
	self.Texture:Hide()
	self:SaveBarConfig()
	for k,v in ipairs(SpellGroups) do
		self.Groups[v].Header:Show()
	end
	Yata:HideTimerAnchors()	
end

function TotemBar:GetCurrentCallIndex()
	for k, button in ipairs(self.Groups[SPELL_GROUP_CALL].Buttons) do
		if button:GetAttribute("position") == 0 then
			return k - 1
		end
	end
	
	return 0
end

function TotemBar:ConfigureBarDisplay(gapValue, orientationValue, visibleValue)
	gapValue = gapValue or Yata.db.profile.ButtonGap
	orientationValue = orientationValue or Yata.db.profile.BarOrientation
	visibleValue = visibleValue or Yata.db.profile.VisibleButtons
	
	Yata.db.profile.ButtonGap = gapValue
	Yata.db.profile.BarOrientation = orientationValue
	Yata.db.profile.VisibleButtons = visibleValue
	
	if Yata.db.profile.ButtonSkin == "Blizzard" then
		gapValue = gapValue + 4
	end
	
	local buttonSize =  36 + gapValue
	
	for k, spellgroup in ipairs(SpellGroups) do
		local header = self.Groups[spellgroup].Header
		header:SetAttribute("buttongap",gapValue)
		header:SetWidth(36)
		header:SetHeight(36)
		header:SetAttribute("visiblebuttons", visibleValue)
		header:SetAttribute("orientation",orientationValue)
		header:SetAttribute("position",k-1)
		header:SetAttribute("show",false)
	end
	
	self:SetWidth(#SpellGroups * buttonSize)

	self:PositionButtons()

	for k,v in ipairs(self.Groups.All) do
		 if Yata.db.profile.ButtonSkin == "Blizzard" then
			v.Duration:SetPoint("TOP",v.Parent,"BOTTOM",0,-4)
		 else
			v.Duration:SetPoint("TOP",v.Parent,"BOTTOM",0,0)
		end
	end
		
	self:ApplyTotemSet()
end

function TotemBar:UpdateAllButtons(spellgroup)
	for k, button in ipairs(self.Groups[spellgroup].Buttons) do
		button:SetCallIndicators()
	end
end

function TotemBar:UpdateCallIndicators()
	for k, button in ipairs(self.Groups.All) do
		button:SetCallIndicators()
	end
end

function Yata:LIBKEYBOUND_ENABLED()
	for i,b in ipairs(SpellGroups) do
		Yata.Bar.Groups[b].Header:SetAttribute("visiblebuttons",10)
		Yata.Bar.Groups[b].Header:SetAttribute("show",true)
		Yata.Bar.Groups[b].Proxy:Show()
	end
	Yata.keyBoundMode = true
end

function Yata:LIBKEYBOUND_DISABLED()
	Yata.keyBoundMode = nil
	for i,b in ipairs(SpellGroups) do
		Yata.Bar.Groups[b].Header:SetAttribute("visiblebuttons",Yata.db.profile.VisibleButtons)
		Yata.Bar.Groups[b].Header:SetAttribute("show",false)
		Yata.Bar.Groups[b].Proxy:Hide()
	end
	self.Bar:SetProxyBinding()
end

function Yata:LIBKEYBOUND_MODE_COLOR_CHANGED()
end

function TotemBar:SetProxyBinding()
	for i,b in ipairs(SpellGroups) do
		local key = GetBindingKey(format("CLICK %s:LeftButton", self.Groups[b].Proxy.Name))
		self.Groups[b].Header:SetAttribute("proxybinding",key)
	end
end

function TotemBar:EditMacro(force, old,new)
	local num = GetNumMacros()
	local macroindex = GetMacroIndexByName("YataTotemStomp")
	if force or macroindex == 0 and num < 36 then
		local macro = "#showtooltip\n/castsequence reset=combat/"..Yata.db.profile.MacroResetKey.." "
		local totems = {}
		local set = Yata:GetTotemSet()
		for k,v in ipairs(set.GroupOrder) do
			if v ~= SPELL_GROUP_IMBUE and v~= SPELL_GROUP_CALL then
				for i,b in ipairs(self.Groups[v].Buttons) do
					if b:GetAttribute("position") == 0 then
						table.insert(totems,b.Totem.Name)
						break
					end
				end
			end
		end
		for k,v in ipairs(totems) do
			if totems[k] and totems[k+1] then
				macro = string.format("%s%s, ",macro,v)
			else
				macro = string.format("%s%s",macro,v)
			end
		end
		local numIcons = GetNumMacroIcons()
		local iconid = 0
		for i=1,numIcons do
			if GetMacroIconInfo(i) == "Interface\\Icons\\INV_Misc_QuestionMark" then
				iconid = i
				break
			end
		end	
		if force and macroindex > 0 then		
			EditMacro(macroindex, "YataTotemStomp", iconid, macro, 1)
		else
			CreateMacro("YataTotemStomp",iconid,macro,1,1)
		end
	elseif macroindex > 0 and old and new then
		local name, texture, macro, isLocal = GetMacroInfo(macroindex)
		macro = string.gsub(macro,old,new)
		EditMacro(macroindex, name, texture, macro, isLocal);
	end	
end